
/******************************
 * perms.css
 ******************************/

div.field-couple.permissions div.field p.group {
    font-weight: bold;
}

div.field-couple.permissions div.field label {
    float: none;
    font-weight: normal;
}

div.field-couple.permissions div.field label input {
    margin-right: 10px;
}
