
/******************************
 * General
 ******************************/

* {
    margin: 0px;
}

html, body {
    font-family: sans-serif;
    font-size: 11pt;
}

a {
    text-decoration: none;
}

a:hover {
    text-decoration: underline;
}

h1 {
    margin-bottom: 15px;
}

h2 {
    font-size: 12pt;
    margin: 20px auto 10px auto;
}

li {
    line-height: 2em;
}

p {
    margin-bottom: 5px;
}

.left {
    float: left;
    text-align: left;
}

.right {
    float: right;
    text-align: right;
}

.wrapper {
    overflow: auto;
}

div.buttons {
    clear: both;
    margin-top: 10px;
}

/* div.controls { */
/*     font-weight: bold; */
/*     margin: 10px auto; */
/* } */

/* div.controls div { */
/*     margin: 5px; */
/* } */

/* div.controls label { */
/*     display: block; */
/*     float: left; */
/*     width: 120px; */
/* } */

div.dialog {
    display: none;
}

div.flash-message {
    background-color: #dddddd;
    margin-bottom: 8px;
    padding: 3px;
}

div.error {
    color: #dd6666;
    font-weight: bold;
    margin-bottom: 10px;
}

/* td.right { */
/*     float: none; */
/* } */

/* table.wrapper td.right { */
/*     vertical-align: bottom; */
/* } */


/******************************
 * Main Layout
 ******************************/

html, body, #container {
    height: 100%;
}

body > #container {
    height: auto;
    min-height: 100%;
}

#container {
    margin: 0 auto;
    width: 1000px;
}

#header {
    border-bottom: 1px solid #000000;
    overflow: auto;
}

#body {
    padding-top: 15px;
    padding-bottom: 5em;
}

#footer {
    margin-top: -4em;
    text-align: center;
}


/******************************
 * Header
 ******************************/

#header h1 {
    margin: 0px 5px 10px 5px;
}

#login {
    margin: 8px 20px auto auto;
}

#user-menu {
    float: left;
}

#home-link {
    font-weight: bold;
}

#header-links {
    float: right;
    text-align: right;
}

#main-menu {
    border-top: 1px solid black;
    clear: both;
    font-weight: bold;
}

#main-menu li {
    display: inline;
    margin-right: 15px;
}
