
/******************************
 * Autocomplete
 ******************************/

div.autocomplete {
    border: 1px solid #000000;
    margin-top: 5px;
}

div.autocomplete div {
    background-color: #dddddd;
    margin: 0px;
    padding: 2px 5px;
}

div.autocomplete strong {
    margin: 0px 1px;
}

div.autocomplete .selected {
    cursor: pointer;
    background-color: #aaaaaa;
}
