
/******************************
 * Grid Header
 ******************************/

table.grid-header {
    padding-bottom: 5px;
    width: 100%;
}


/******************************
 * Form (Filters etc.)
 ******************************/

table.grid-header td.form {
    vertical-align: bottom;
}


/******************************
 * Context Menu
 ******************************/

table.grid-header td.context-menu {
    vertical-align: top;
}

table.grid-header td.context-menu ul {
    list-style-type: none;
    margin: 0px;
    text-align: right;
}


/******************************
 * Tools
 ******************************/

table.grid-header td.tools {
    text-align: right;
    vertical-align: bottom;
}


/******************************
 * Grid
 ******************************/

div.grid {
    clear: both;
}

div.grid table {
    border-top: 1px solid black;
    border-left: 1px solid black;
    border-collapse: collapse;
    font-size: 9pt;
    line-height: normal;
    white-space: nowrap;
}

div.grid.full table {
    width: 100%;
}

div.grid table th,
div.grid table td {
    border-right: 1px solid black;
    border-bottom: 1px solid black;
    padding: 2px 3px;
}

div.grid table th.sortable a {
    display: block;
    padding-right: 18px;
}

div.grid table th.sorted {
    background-position: right center;
    background-repeat: no-repeat;
}

div.grid table th.sorted.asc {
    background-image: url(../img/sort_arrow_up.png);
}

div.grid table th.sorted.desc {
    background-image: url(../img/sort_arrow_down.png);
}

div.grid table tbody td {
    text-align: left;
}

div.grid table tbody td.center {
    text-align: center;
}

div.grid table tbody td.right {
    float: none;
    text-align: right;
}

div.grid table tr.odd {
    background-color: #e0e0e0;
}

/* div.grid table thead th.checkbox, */
/* div.grid table tbody td.checkbox { */
/*     text-align: center; */
/*     vertical-align: middle; */
/*     width: 15px; */
/* } */

div.grid table tbody tr.hovering {
    background-color: #bbbbbb;
}

div.grid table.hoverable tbody tr {
    cursor: default;
}

div.grid.clickable table tbody tr,
div.grid table.selectable tbody tr,
div.grid table.checkable tbody tr {
    cursor: pointer;
}

div.grid.clickable table tbody tr td.noclick {
    cursor: default;
    text-align: center;
    width: 18px;
}

div.grid table tbody tr td.noclick.edit,
div.grid table tbody tr td.noclick.delete {
    background-repeat: no-repeat;
    background-position: center;
    cursor: pointer;
    min-width: 18px;
}

div.grid table tbody tr td.noclick.edit {
    background-image: url(../img/edit.png);
}

div.grid table tbody tr td.noclick.delete {
    background-image: url(../img/delete.png);
}

/* div.grid table.selectable tbody tr.selected, */
/* div.grid table.checkable tbody tr.selected { */
/*     background-color: #666666; */
/*     color: white; */
/* } */

div.pager {
    margin-bottom: 20px;
    margin-top: 5px;
}

div.pager p {
    font-size: 10pt;
    margin: 0px;
}

div.pager p.showing {
    float: left;
}

div.pager #grid-page-count {
    font-size: 8pt;
}

div.pager p.page-links {
    float: right;
}
